/*********************************************************************
 *
 * File:         users.c
 *
 * Description:  NCS users server
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         12-17-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "server.h"
#include "global.h"

typedef struct _user
{
  Participant part;
  struct _user *next;
} User;

User *head=NULL;
int number = 0;

extern FILE *log;

int comp_parts(Participant *part1, Participant *part2)
{
  if(   (strcmp(part1->user, part2->user) == 0)
     && (part1->host == part2->host))
    return 1;
  else
    return 0;
}


int find_user(char *data)
{
  User *pom;
  int found = 0;
  Participant *part;

  part = (Participant *)data;

  pom = head;
  if(pom == NULL)
    return 0;

  if(comp_parts(&pom->part, part))
    return 1;

  while((pom->next != NULL) && (!found))
    {
      found = comp_parts(&pom->next->part, part);
      if(!found)
	pom = pom->next;
    }
  return found;  
}

void add_user(char *data)
{
  User *pom;

  if(find_user(data))
    {
      fprintf(log, "User already in database\n");
      return;
    }
  if(head == NULL)
    {
      head = (User *)malloc(sizeof(User));
      pom = head;
    }
  else
    {
      pom = head;
      while(pom->next != NULL)
	pom = pom->next;
      pom->next = (User *)malloc(sizeof(User));
      pom = pom->next;
    }

  memcpy(&pom->part, data, sizeof(Participant));
  pom->next = NULL;
  number++;
}

void del_user(char *data)
{
  User *pom, *tmp;
  Participant *part;
  int found = 0;

  part = (Participant *)data;
  pom = head;
  if(pom != NULL)
    {
      if((head->next == NULL) && (comp_parts(part, &head->part)))
	{
	  free(head);
	  head = NULL;
	  number--;
	}
      else
	{
	  if(comp_parts(&pom->part, part))
	    {
	      head = pom->next;
	      free(pom);
	      number--;
	      return;
	    }
	  while((pom->next != NULL) && (!found))
	    {
	      found = comp_parts(&pom->next->part, part);
	      if(!found)
		pom = pom->next;
	    }
	  if(found)
	    {
	      tmp = pom->next;
	      pom->next = pom->next->next;
	      free(tmp);
	      number--;
	    }
	}	    
    }
}

void send_users(int socket, Participant *rec)
{
  ServerData *data;
  int size, i=0;
  User *pom;

  if(number > 0)
    size = sizeof(ServerData) + ((number-1) * sizeof(NCSparticipant));
  else
    size = sizeof(ServerData);

  data = (ServerData *)malloc(size);
  pom = head;
  fprintf(log, "Send %d users to %s@%d\n", number, rec->user, rec->host);
  while(pom != NULL)
    {
      strcpy(data->parts[i].user, pom->part.user);
      getHostName(pom->part.host, data->parts[i].host);
      fprintf(log, "    User: %s@%s\n", data->parts[i].user, data->parts[i].host);
      i++;
      pom = pom->next;
    }
  data->numMembers = number;  
  
  sendServerMessage(socket, indServerUsers, (char *)data, size);
}








